(* ========================================================================= *)
(*                                                                           *)
(*   Quantum optics library: single mode electromagnetic field.              *)
(*                           ([Part of] Layer 3)                             *)
(*                                                                           *)
(*    (c) Copyright, Mohamed Yousri Mahmoud, Vincent Aravantinos, 2012-2013  *)
(*                   Hardware Verification Group,                            *)
(*                   Concordia University                                    *)
(*                                                                           *)
(*           Contact: <mosolim@ece.concordia.ca>, <vincent@ece.concordia.ca> *)
(*                                                                           *)
(* Last update: Jan 2014                                                     *)                                                                           
(* ========================================================================= *)




(*****************************************************************************)
(* SINGLE-MODE DEFINITION                                                    *)
(*****************************************************************************)

let is_qst = new_definition
  `is_qst x <=>
   x IN sq_integrable /\ r_inprod x x = Cx(&1)`;;

   let IS_QST_IN_SQ = prove
(`!x. is_qst x ==> x IN sq_integrable`, SIMP_TAC[is_qst]);;

let IS_QST_UNITY = prove
(`!x. is_qst x ==> r_inprod x x = Cx(&1)`, SIMP_TAC[is_qst]);;

let get_qst =  new_definition
 `get_qst v = Cx(inv(cfun_norm r_inprod v)) % v`;; 
 
let NORMED_IS_QST = prove
( `!v. v IN sq_integrable /\ ~(cfun_norm r_inprod v = &0) 
   ==> is_qst  (get_qst v)`,
  REWRITE_TAC[get_qst;is_qst] THEN
  ASSUME_TAC SQ_INTEGRABLE_INNER_SPACE THEN 
  IMP_REWRITE_TAC[INPROD_RSMUL;INPROD_LSMUL;INNER_SPACE_SMUL] THEN
  REWRITE_TAC[COMPLEX_MUL_ASSOC;CNJ_CX;GSYM CX_MUL;GSYM REAL_INV_MUL; GSYM REAL_POW_2] THEN
  IMP_REWRITE_TAC[CFUN_NORM_POW2] THEN
  TARGET_REWRITE_TAC [MESON[REAL] `real z ==> x * z = x * Cx(Re z)`]
  REAL_OF_COMPLEX_RE THEN
  IMP_REWRITE_TAC[CFUN_NORM_INPROD_0;INPROD_SELF_REAL;REAL_OF_COMPLEX_RE;
  GSYM CX_MUL;REAL_MUL_LINV] THEN  MESON_TAC[]);;

let DETERMINISTIC_MEASURE_EIGEN_STATE = prove
  (`!op s st v.
     is_unbounded_linear_cop s op 
       /\ is_qst st /\ is_eigen_pair_unbounded s op (st,v)
        ==> expectation r_inprod st op = v`,
  SIMP_TAC[is_qst;is_eigen_pair_unbounded;expectation] 
  THEN MESON_TAC[INPROD_RSMUL;COMPLEX_MUL_RID;SQ_INTEGRABLE_INNER_SPACE]);;

new_type_abbrev("bqs",`:real->complex`);;
new_type_abbrev("qop",`:bqs->bqs`);;
new_type_abbrev("sm",`:qop#qop#real#bqs`);; 
  

let FORALL_SM_THM = prove
  (`!P. (!sm:sm. P sm) <=> !qsys omega vac. P (qsys,omega,vac)`,
  REWRITE_TAC[FORALL_PAIR_THM]);;

let FORALL_SM_THM = prove
  (`!P. (!sm:sm. P sm) <=> !ad a f v. P (ad,a,f,v)`,
  REWRITE_TAC[FORALL_PAIR_THM]);;



let w_sm = new_definition 
  `w1 ((ad,a,f,v):sm) = f`;;

let planck = new_specification ["planck"]
  (prove(`?x. &0 < x`, EXISTS_TAC `&1` THEN REAL_ARITH_TAC));;

let cr_sm = new_definition
  `cr ((ad,a,f,v):sm) = ad`;;

let anh_sm = new_definition
  `anh ((ad,a,f,v):sm) = a`;;
let vac_sm = new_definition
  `vac ((ad,a,f,v):sm) = v`;;
let h_sm = new_definition
  `h (sm:sm) = Cx(planck * w1 sm) %
          (cr sm  ** anh sm + Cx(inv(&2)) % I)`;;
let dcr = new_definition
 ` dcr (sm:sm) = {x| x IN sq_integrable /\ cr sm x IN sq_integrable}`;;

let danh = new_definition
 ` danh (sm:sm) = {x| x IN sq_integrable /\ anh sm x IN sq_integrable}`;;

let dh = new_definition
 `dh2 (sm:sm) = {x|x IN (danh sm) /\ (anh sm x) IN (danh sm)}`;;
 
 
let is_sm = new_definition 
  `is_sm6 (sm:sm) <=>
    &0 < w1 sm /\ dcr sm = danh sm /\ (!n. ((cr sm) pow n) (vac sm) IN sq_integrable) /\
    is_hermitian_unbounded (sq_integrable, r_inprod) (danh sm) (dcr sm) (anh sm) (cr sm)
    /\ (!st. st IN (dh2 sm) ==> (anh sm com cr sm) st = st) /\ is_qst (vac sm) /\ 
	  is_eigen_pair_unbounded (dh2 sm) (h sm) (vac sm,Cx(planck * (w1 sm)/ &2))`;;
	
let FORALL_IS_SM_THM = prove
  (`!P. (!sm. is_sm6 sm ==> P sm)
    <=> (!a adag w' v.
	  &0 < w' /\ dcr (adag,a,w',v) = danh (adag,a,w',v) /\ 
	  (!n. ((cr (adag,a,w',v)) pow n) (vac (adag,a,w',v)) IN sq_integrable) /\
      is_hermitian_unbounded (sq_integrable, r_inprod) (danh (adag,a,w',v)) (dcr (adag,a,w',v)) a adag
    /\ (!st. st IN (dh2 (adag,a,w',v)) ==> (a com adag) st = st) /\ is_qst v /\ 
	  is_eigen_pair_unbounded (dh2 (adag,a,w',v)) (h (adag,a,w',v)) (v,Cx(planck * (w')/ &2))
	  ==> P (adag,a,w',v))`,
  GEN_TAC
  THEN REWRITE_TAC[FORALL_SM_THM;is_sm;
    FORALL_AND_THM;GSYM FUN_EQ_THM;anh_sm;cr_sm;vac_sm;w_sm]
  THEN MESON_TAC[]);;

let OMEGA_LT = prove
(`!sm.
	 is_sm6 sm ==> &0 < w1 sm `, REWRITE_TAC[is_sm] THEN 
  MESON_TAC[]);;
 
let CREAT_CLAUSER = prove
(`!sm n. is_sm6 sm ==>  ((cr sm) pow n) (vac sm) IN sq_integrable`,
  REWRITE_TAC[is_sm] THEN 
  MESON_TAC[]);;
  
let A_HERM_A = prove
  (`!sm. is_sm6 sm ==> 
  is_hermitian_unbounded (sq_integrable, r_inprod) (danh sm) (dcr sm) (anh sm) (cr sm)`,  
  MESON_TAC[is_sm]);;

let HERM_A_A = ONCE_REWRITE_RULE [HERM_SYM_UNBOUNDED] A_HERM_A;;  
let DOMAINS_EQ = prove
(`!sm. is_sm6 sm ==> (danh sm) = (dcr sm)`, MESON_TAC[is_sm]);;


let A_HERM_A_COMMUTATOR = prove
(`!sm st. is_sm6 sm /\  st IN (dh2 sm) ==>
          ((anh sm) com (cr sm)) st = st`,
 MESON_TAC[is_sm]);;		  

let SM_H_PROP = prove
  (`!sm. is_sm6 sm ==> 
  is_symmetric (sq_integrable,r_inprod) (dh2 sm)(h sm)`,  
  REWRITE_TAC[dh;h_sm] THEN SYMMETRIC_TAC
  THEN ASM_SIMP_TAC[SIMP_RULE [DOMAINS_EQ] HERM_A_A;DOMAINS_EQ;REAL_CX]);;
  
let SM_H_LINEAR = prove
  (`!sm. is_sm6 sm ==> is_unbounded_linear_cop (dh2 sm)(h sm)`,
  MESON_TAC[SYMMETRIC_IS_LINCOP;SM_H_PROP;SQ_INTEGRABLE_INNER_SPACE]);;


let SM_VAC_QST = prove
  (`!sm. is_sm6 sm ==> is_qst (vac sm)`, 
  MESON_TAC[is_sm]);;
  
let SM_VAC_H_PAIR = prove
  (`!sm. is_sm6 sm ==>
    is_eigen_pair_unbounded (dh2 sm)
	  (h sm) (vac sm,Cx(planck * (w1 sm)/ &2))`,
    MESON_TAC[is_sm]);;	  
	
let VAC_QST_PROP = LET_RULE_L[is_qst] SM_VAC_QST;;


	 
let A_HERM_A_LINEAR = prove
    (`!sm. is_sm6 sm ==>
    is_unbounded_linear_cop (danh sm)  (anh sm) 
	/\ is_unbounded_linear_cop (dcr sm) (cr sm)`,
  MESON_TAC[REWRITE_RULE[is_hermitian_unbounded] A_HERM_A;SQ_INTEGRABLE_INNER_SPACE]);;



let ANNIHL_En  = prove
  (`!sm st v.  
         is_sm6 sm /\ ~(anh sm st = cfun_zero) 
		 /\ (anh sm st IN (dh2 sm)) ==>
         is_eigen_pair_unbounded (dh2 sm) (h sm) (st,v) ==>
         is_eigen_pair_unbounded (dh2 sm) (h sm)  (anh sm st,v - Cx(planck*(w1 sm)))`,
  REWRITE_TAC[is_eigen_pair_unbounded;h_sm;dh;IN_ELIM_THM;is_qst] THEN SIMP_TAC[]
  THEN REPEAT STRIP_TAC THEN  POP_ASSUM (fun th1 ->  POP_ASSUM (MP_TAC o C MATCH_MP th1))
 THEN IMP_REWRITE_TAC [dh;IN_ELIM_THM;COP_SMUL_THM;COP_ADD_THM;COP_MUL_THM;REWRITE_RULE 
 [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> y = x- z`;GSYM IMP_IMP] (Pa.SPECL[ "sm";"anh sm st" ] A_HERM_A_COMMUTATOR)]
 THEN IMP_REWRITE_TAC[CFUN_ARITH ` (x:cfun) + y = z <=> x = z -y`
  ;COP_MUL_THM;COP_ADD_THM;CFUN_SMUL_DIS;CFUN_EQ_SMUL_LCANCEL2;CX_INJ;planck
  ;REAL_FIELD`&0< x /\ &0< y ==> ~(x*y = &0) `;OMEGA_LT;ULINCOP_SUB
  ;REWRITE_RULE [is_unbounded_linear_cop] A_HERM_A_LINEAR]
  THEN ASM_SIMP_TAC[I_THM;GSYM COP_I_SCALAR;COP_SMUL_THM;CFUN_SMUL_DIS;CFUN_SUB_LDISTRIB ;CFUN_SMUL_ASSOC]
  THEN  IMP_REWRITE_TAC[COMPLEX_MUL_LINV;CX_INJ;planck;REAL_FIELD`&0< x /\ &0< y ==> ~(x*y = &0)`;COMPLEX_MUL_LID;
  COMPLEX_ADD_LDISTRIB;COMPLEX_SUB_LDISTRIB;OMEGA_LT]
  THEN DISCH_TAC THEN Pa.EXISTS_TAC "danh sm" THEN IMP_REWRITE_TAC[CFUN_SUBSPACE_SMUL;REWRITE_RULE [is_unbounded_linear_cop] A_HERM_A_LINEAR]
  THEN ASM_SIMP_TAC[A_HERM_A_LINEAR] THEN CFUN_ARITH_TAC);;

let CREAT_En  = prove
  (`!sm st v.  
         is_sm6 sm /\ is_qst st
		 /\ (cr sm st IN (dh2 sm)) ==>
         is_eigen_pair_unbounded (dh2 sm) (h sm) (st,v) ==>
         is_eigen_pair_unbounded (dh2 sm) (h sm)  (cr sm st,v + Cx(planck*(w1 sm)))`,
   
  REWRITE_TAC[is_eigen_pair_unbounded;h_sm;dh;IN_ELIM_THM;is_qst] THEN SIMP_TAC[]
  THEN REPEAT STRIP_TAC THENL[
  POP_ASSUM (fun th1 ->  POP_ASSUM (MP_TAC o C MATCH_MP th1))
  THEN IMP_REWRITE_TAC [dh;IN_ELIM_THM;COP_SMUL_THM;COP_ADD_THM;COP_MUL_THM;REWRITE_RULE 
  [commutator;COP_SUB_THM;COP_MUL_THM;
  CFUN_ARITH `(x:cfun) - y = z <=> x =  z +y `;GSYM IMP_IMP] A_HERM_A_COMMUTATOR]
 THEN IMP_REWRITE_TAC[CFUN_ARITH ` (x:cfun) + y = z <=> x = z -y`
  ;COP_MUL_THM;COP_ADD_THM;CFUN_SMUL_DIS;CFUN_EQ_SMUL_LCANCEL2;CX_INJ;planck
  ;REAL_FIELD`&0< x /\ &0< y ==> ~(x*y = &0) `;OMEGA_LT;ULINCOP_ADD;
  DOMAINS_EQ;CFUN_SUBSPACE_SUB;CFUN_SUBSPACE_SMUL;
  REWRITE_RULE [is_unbounded_linear_cop] A_HERM_A_LINEAR]
  THEN ASM_SIMP_TAC[I_THM;GSYM COP_I_SCALAR;COP_SMUL_THM;CFUN_SMUL_DIS;CFUN_SUB_LDISTRIB ;CFUN_SMUL_ASSOC]
  THEN  IMP_REWRITE_TAC[COMPLEX_MUL_LINV;CX_INJ;planck;REAL_FIELD`&0< x /\ &0< y ==> ~(x*y = &0)`;COMPLEX_MUL_LID;
  COMPLEX_ADD_LDISTRIB;COMPLEX_SUB_LDISTRIB;OMEGA_LT] THEN 
  DISCH_TAC THEN IMP_REWRITE_TAC[Pa.SPEC "(inv (Cx (planck * w1 sm)) * v) % st" ULINCOP_SUB] THEN
  Pa.EXISTS_TAC "dcr sm" THEN IMP_REWRITE_TAC[CFUN_SUBSPACE_SMUL;ULINCOP_SUBSPACE;  REWRITE_RULE [is_unbounded_linear_cop]A_HERM_A_LINEAR]
  THEN ASM_SIMP_TAC[A_HERM_A_LINEAR] THEN  Pa.EXISTS_TAC "st" THEN  
  ASM_SIMP_TAC[] THEN  Pa.EXISTS_TAC "st" THEN ASM_SIMP_TAC[]
  THEN CFUN_ARITH_TAC;ALL_TAC] THEN POP_ASSUM MP_TAC THEN REWRITE_TAC[]
  THEN   MATCH_MP_TAC INPROD_NOT_ZERO 
  THEN MAP_EVERY Pa.EXISTS_TAC ["sq_integrable";"r_inprod"] THEN
  ASM_SIMP_TAC[GSYM EQUV_ZERO;SQ_INTEGRABLE_INNER_SPACE] THEN CONJ_TAC
  THENL[RULE_ASSUM_TAC(REWRITE_RULE[danh;IN_ELIM_THM]) THEN ASM_SIMP_TAC[];ALL_TAC]
   THEN IMP_REWRITE_TAC[Pa.SPECL ["cr sm";"anh sm";"dcr sm";"danh sm"] (tem HERM_ITSELF_UNBOUNDED);HERM_A_A]
   THEN IMP_REWRITE_TAC [dh;IN_ELIM_THM;REWRITE_RULE 
 [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> x =  z +y `;GSYM IMP_IMP] A_HERM_A_COMMUTATOR]
 THEN IMP_REWRITE_TAC[tem INPROD_ADD_RDIST]
 THEN  IMP_REWRITE_TAC[Pa.SPECL ["anh sm";"cr sm";"danh sm";"dcr sm"] (tem (GSYM HERM_ITSELF_UNBOUNDED));A_HERM_A]
 THEN ASM_SIMP_TAC[GSYM DOMAINS_EQ] THEN 
STRIP_TAC THENL[POP_ASSUM (fun th1 ->  POP_ASSUM (MP_TAC o C MATCH_MP th1)) THEN 
ASM_SIMP_TAC[DOMAINS_EQ;dcr;IN_ELIM_THM];ALL_TAC]
THEN 
 REWRITE_TAC[COMPLEX_FIELD `Cx(&1) + y = Cx(&0) <=> y =  -- Cx(&1)`]
    THEN STRIP_TAC	
   	THEN  ASSUME_TAC (tem INPROD_SELF_POS)
	THEN  POP_ASSUM( MP_TAC o (SPEC `anh sm st`))
	THEN RULE_ASSUM_TAC(REWRITE_RULE[danh;IN_ELIM_THM]) THEN 
	ASM_SIMP_TAC[REAL_OF_COMPLEX_CX;GSYM CX_NEG]
	THEN REAL_ARITH_TAC);;
	

let MINIMUM_ENERGY = prove
  (`!sm st.
      is_sm6 sm/\ st IN (dh2 sm) /\ is_qst st ==>
       planck * (w1 sm) / &2 <= real_of_complex (expectation r_inprod st (h sm))`,
  REWRITE_TAC[expectation;is_qst;h_sm] THEN REPEAT STRIP_TAC THEN
  REWRITE_TAC[CFUN_ADD_LDISTRIB;COP_ADD_THM;COP_SMUL_THM;COP_MUL_THM;I_THM;CFUN_SMUL_ASSOC]
  THEN  IMP_REWRITE_TAC[tem INPROD_ADD_LDIST; tem INPROD_RSMUL]
  THEN IMP_REWRITE_TAC[Pa.SPECL ["cr sm";"anh sm";"dcr sm";"danh sm"] (tem HERM_ITSELF_UNBOUNDED);HERM_A_A]
  THEN IMP_REWRITE_TAC[REAL_OF_COMPLEX_MUL;REAL_OF_COMPLEX_ADD]
  THEN IMP_REWRITE_TAC[REAL_MUL;REAL_CX;REAL_OF_COMPLEX_CX;REAL_MUL_RID]
  THEN IMP_REWRITE_TAC[INPROD_SELF_REAL;CFUN_SUBSPACE_SMUL] THEN
  REWRITE_TAC[REAL_LE_ADDL;GSYM real_div;GSYM REAL_MUL_ASSOC] THEN
  REPEAT (POP_ASSUM MP_TAC) THEN SIMP_TAC[dh;IN_ELIM_THM]
  THEN SIMP_TAC [GSYM DOMAINS_EQ] THEN SIMP_TAC [DOMAINS_EQ;dcr;IN_ELIM_THM]
  THEN REPEAT STRIP_TAC THENL[MATCH_MP_TAC (MESON[REAL_LE_MUL;REAL_LT_MUL;REAL_LE_LT]
      `&0 <= z /\ &0 < x /\ &0 < y ==> &0 <= x * y * z`) THEN
  IMP_REWRITE_TAC[ planck;INPROD_SELF_POS;OMEGA_LT;SQ_INTEGRABLE_INNER_SPACE];
  Pa.EXISTS_TAC "sq_integrable" THEN ASM_SIMP_TAC[SQ_INTEGRABLE_INNER_SPACE]
  ;ASM_MESON_TAC[SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_IS_SUBSPACE]]);;
(****************************************************************************)
(* PHOTON OPERATOR                                                          *)
(****************************************************************************)
	 
let phn_sm = new_definition 
  `phn sm  = cr sm ** anh sm`;;

let N_SYMMETRIC = prove
  (`!sm. is_sm6 sm ==> is_symmetric (sq_integrable,r_inprod) (dh2 sm)
  (phn sm)`,
  REWRITE_TAC[phn_sm;is_symmetric;dh]
  THEN REPEAT STRIP_TAC THEN MATCH_MP_TAC MUL_HERM_UNBOUNDED
  THEN ASM_MESON_TAC[A_HERM_A;HERM_SYM_UNBOUNDED;DOMAINS_EQ]);;
  


let N_LINEAR = prove
  (`!sm. is_sm6 sm ==>  is_unbounded_linear_cop (dh2 sm)
  (phn sm)`,
 MESON_TAC[SYMMETRIC_IS_LINCOP;N_SYMMETRIC;SQ_INTEGRABLE_INNER_SPACE]);;

let H_PHOTON_N = prove
  (`!sm. is_sm6 sm ==>
	     h sm = Cx(planck * w1 sm) % (phn sm + Cx(inv(&2)) % I)`,
   SIMP_TAC[h_sm;phn_sm]);;

let EG_FUN_H_N = prove
  (`!sm st v. is_sm6 sm ==>
        (is_eigen_pair_unbounded (dh2 sm) (h sm) (st,v) <=> 
        is_eigen_pair_unbounded
		(dh2 sm) (phn sm)  (st,Cx(inv(planck * w1 sm)) * v - Cx(inv(&2))))`,
		
   SIMP_TAC[is_eigen_pair_unbounded;N_LINEAR;SM_H_LINEAR] 
   THEN IMP_REWRITE_TAC[H_PHOTON_N;CFUN_EQ_SMUL_LCANCEL2;COP_SMUL_THM;COP_ADD_THM;planck
   ;REAL_FIELD`&0< x /\ &0< y ==> ~(x*y = &0)`;CX_INV;CX_INJ;OMEGA_LT;I_THM]
   THEN MESON_TAC[CFUN_SMUL_ASSOC;CFUN_EQ_RSUB;CFUN_SUB_RDISTRIB]);;
    
let EG_FUN_N_H = prove
  (`!sm st v. is_sm6 sm ==>
        (is_eigen_pair_unbounded (dh2 sm) (phn sm) (st,v) <=> 
        is_eigen_pair_unbounded
		(dh2 sm) (h sm)  (st,Cx(planck * w1 sm) * (v + Cx(inv(&2)))))`,
  SIMP_TAC[EG_FUN_H_N;GSYM CX_MUL;
    COMPLEX_MUL_ASSOC]
  THEN REPEAT STRIP_TAC THEN MP_REWRITE_TAC [REAL_MUL_LINV]
  THEN GCONV_TAC COMPLEX_POLY_CONV
  THEN ASM_MESON_TAC[OMEGA_LT;planck;REAL_POS_NZ;REAL_ENTIRE]);;
 
 
let ANNIHL_PHOTON  = prove
  (`!sm. is_sm6 sm ==>
	   is_eigen_pair_unbounded (dh2 sm) (phn sm) (st,v) /\ (anh sm st IN (dh2 sm)) /\ 
	   ~(anh sm st = cfun_zero) ==>
         is_eigen_pair_unbounded (dh2 sm) (phn sm) (anh sm st,v - Cx(&1))`,
		 
  SIMP_TAC[EG_FUN_N_H;COMPLEX_ADD_LDISTRIB
 ;COMPLEX_SUB_LDISTRIB;COMPLEX_MUL_RID]
 THEN MESON_TAC[COMPLEX_FIELD `((a:complex)-b)+c = (a+c)-b`;ANNIHL_En]);;

let CREAT_PHOTON = prove
(`!sm. is_sm6 sm ==>
	   is_eigen_pair_unbounded (dh2 sm) (phn sm) (st,v) /\ (cr sm st IN (dh2 sm)) /\ 
	   is_qst st ==>
         is_eigen_pair_unbounded (dh2 sm) (phn sm) (cr sm st,v + Cx(&1))`,
		 
		 SIMP_TAC[EG_FUN_N_H;COMPLEX_ADD_LDISTRIB;COMPLEX_MUL_RID]
 THEN MESON_TAC[COMPLEX_FIELD `((a:complex)+b)+c = (a+c)+b`; CREAT_En]);;
 
  
let VAC_ZERO_PHOTON = prove
 (`!sm. is_sm6 sm ==>
	   is_eigen_pair_unbounded (dh2 sm) (phn sm) (vac sm,Cx (&0))`,
	SIMP_TAC[EG_FUN_N_H;COMPLEX_ADD_LID;
	GSYM CX_MUL;real_div;REAL_MUL_ASSOC;is_sm]);;
        
let NORM_A_VAC = prove
(`!sm. is_sm6 sm ==>
	  r_inprod (anh sm  (vac sm) ) (anh sm  (vac sm)) = Cx(&0)`, 
  
  REPEAT STRIP_TAC THEN
  IMP_REWRITE_TAC[GSYM HERM_ITSELF_UNBOUNDED;  A_HERM_A_LINEAR;ULINCOP_SMUL]THEN
  MAP_EVERY Pa.EXISTS_TAC ["cr sm ";"sq_integrable";"dcr sm";"danh sm"]
  THEN ASM_SIMP_TAC[GSYM COP_MUL_THM;GSYM phn_sm] THEN
   IMP_REWRITE_TAC[GSYM expectation;DETERMINISTIC_MEASURE_EIGEN_STATE;N_LINEAR;
   SM_VAC_QST;SQ_INTEGRABLE_INNER_SPACE; VAC_ZERO_PHOTON; HERM_A_A]
  THEN CONJ_TAC THENL[MAP_EVERY Pa.EXISTS_TAC ["dh2 sm";"Cx(&0)"] THEN ASM_SIMP_TAC[N_LINEAR;VAC_ZERO_PHOTON];ALL_TAC]  
  THEN ASM_SIMP_TAC[GSYM DOMAINS_EQ] THEN
  ASM_MESON_TAC[is_eigen_pair_unbounded;dh;IN_ELIM_THM;VAC_ZERO_PHOTON;N_LINEAR] );; 
  
let A_VAC = prove
(`!sm. is_sm6 sm ==>
	  ((anh sm  (vac sm)) equv cfun_zero) r_inprod`, 
	  SIMP_TAC[are_equv;CFUN_SUB_RID;NORM_A_VAC]);;

let fock = define
  `fock sm 0 =  vac sm /\
  fock  sm (SUC n) = 
  get_qst (cr sm (fock sm n))`;;

let CR_INDCR = prove
(`!sm m. is_sm6 sm ==>  (((cr sm) pow m) (vac sm)) IN (dcr sm)`,
SIMP_TAC[dcr;IN_ELIM_THM;GSYM COP_MUL_THM;GSYM cop_pow;CREAT_CLAUSER]);;  
  
let CR_INDANH = prove
(`!sm m. is_sm6 sm ==>  (((cr sm) pow m) (vac sm)) IN (danh sm)`,
SIMP_TAC[DOMAINS_EQ;CR_INDCR]);;
  
let UNNORMALIZED_FOCK = prove(`!sm. is_sm6 sm ==> 
  !n. (cr sm pow n) (vac sm) IN dh2 sm /\
          (phn sm) ((cr sm pow n) (vac sm)) =
          Cx (&n) % (cr sm pow n) (vac sm)`,
  STRIP_TAC THEN DISCH_TAC THEN INDUCT_TAC THENL[
  ASM_SIMP_TAC[cop_pow;FACT;SQRT_1;COMPLEX_INV_1;I_THM] THEN ASM_MESON_TAC[VAC_ZERO_PHOTON;is_eigen_pair_unbounded;N_LINEAR];ALL_TAC] THEN 
  POP_ASSUM MP_TAC THEN REWRITE_TAC[phn_sm;COP_MUL_THM] THEN REPEAT STRIP_TAC THENL[
  ASM_SIMP_TAC[dh;IN_ELIM_THM;CR_INDANH] THEN
  ASM_SIMP_TAC[cop_pow;COP_MUL_THM;REWRITE_RULE [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> x = z +y`;GSYM IMP_IMP] A_HERM_A_COMMUTATOR]
  THEN IMP_REWRITE_TAC[CFUN_SUBSPACE_ADD;CFUN_SUBSPACE_SMUL;CR_INDANH]
  THEN ASM_SIMP_TAC[REWRITE_RULE [is_unbounded_linear_cop]A_HERM_A_LINEAR];ALL_TAC]
  
  THEN ASM_SIMP_TAC[cop_pow;COP_MUL_THM;REWRITE_RULE [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> x = z+y`;GSYM IMP_IMP] A_HERM_A_COMMUTATOR] THEN
  IMP_REWRITE_TAC[REWRITE_RULE [is_unbounded_linear_cop] A_HERM_A_LINEAR;CR_INDCR;CFUN_SUBSPACE_SMUL]
 THEN Pa.EXISTS_TAC " (cr sm pow n) (vac sm)" THEN 
 ASM_SIMP_TAC[CR_INDCR;ADD1;CX_ADD;GSYM REAL_OF_NUM_ADD;CFUN_ADD_RDISTRIB;CFUN_SMUL_LID;CFUN_ADD_SYM]);;
 


let CR_INDH = prove
(`!sm m. is_sm6 sm ==>  (((cr sm) pow m) (vac sm)) IN (dh2 sm)`,
  SIMP_TAC[UNNORMALIZED_FOCK]);;


let NORM_UNORMAIZED_FOCK = prove
(`!sm. is_sm6 sm ==> !m. cfun_norm  r_inprod ((cr sm pow m) (vac sm)) = sqrt(&(FACT m))`,  
STRIP_TAC THEN DISCH_TAC THEN INDUCT_TAC THENL[ 
ASM_SIMP_TAC[fock;cop_pow;FACT;SQRT_1;COMPLEX_INV_1;I_THM
;REAL_OF_COMPLEX_CX;SQRT_1;GSYM ONE;VAC_QST_PROP;cfun_norm] THEN CFUN_ARITH_TAC;ALL_TAC] THEN
POP_ASSUM MP_TAC THEN REWRITE_TAC[cfun_norm;cop_pow;COP_MUL_THM]   
THEN  IMP_REWRITE_TAC[Pa.SPECL ["cr sm";"anh sm";"dcr sm";"danh sm"] (tem HERM_ITSELF_UNBOUNDED);HERM_A_A;CR_INDCR]
   THEN IMP_REWRITE_TAC [REWRITE_RULE 
 [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> x =  z +y `;GSYM IMP_IMP] A_HERM_A_COMMUTATOR;REWRITE_RULE[phn_sm;COP_MUL_THM] UNNORMALIZED_FOCK]
 THEN IMP_REWRITE_TAC[tem INPROD_ADD_RDIST;tem INPROD_LSMUL;CREAT_CLAUSER;CFUN_SUBSPACE_SMUL;INNER_SPACE_IS_SUBSPACE;REAL_OF_COMPLEX_ADD;
 REAL_OF_COMPLEX_MUL;REAL_OF_COMPLEX_CX]
 THEN IMP_REWRITE_TAC[REAL_OF_COMPLEX_ADD;REAL_OF_COMPLEX_CX;CNJ_CX;REAL_CX;SQRT_INJ;REAL_MUL;INPROD_SELF_REAL] THEN 
 REWRITE_TAC[GSYM COP_MUL_THM;GSYM cop_pow;FACT;ADD1;RIGHT_ADD_DISTRIB;GSYM REAL_OF_NUM_ADD;MULT_CLAUSES;GSYM REAL_OF_NUM_MUL]
 THEN  ASM_MESON_TAC[SQ_INTEGRABLE_INNER_SPACE;CREAT_CLAUSER;CR_INDANH;REAL_ARITH `(x:real) +n = n+x`]);;
 
 
 let FOCK_HERM_VAC = prove
 (`!sm. is_sm6 sm ==> !n. fock sm n = inv(Cx(sqrt(& (FACT n)))) % (cr sm pow n) (vac sm)`,
 STRIP_TAC THEN DISCH_TAC THEN INDUCT_TAC THENL[ 
ASM_SIMP_TAC[fock;cop_pow;FACT;SQRT_1;COMPLEX_INV_1;I_THM
;REAL_OF_COMPLEX_CX;SQRT_1;GSYM ONE;VAC_QST_PROP;cfun_norm] THEN CFUN_ARITH_TAC;ALL_TAC] THEN
IMP_REWRITE_TAC[fock;get_qst;CFUN_NORM_SMUL;CREAT_CLAUSER;tem CFUN_NORM_SMUL;REWRITE_RULE[is_unbounded_linear_cop] A_HERM_A_LINEAR]
THEN Pa.EXISTS_TAC " (cr sm pow n) (vac sm)" THEN 
 ASM_SIMP_TAC[CR_INDCR;COP_ARITH `!op:cops. op ((op pow n) x) = (op pow (SUC n)) x`;CREAT_CLAUSER;NORM_UNORMAIZED_FOCK;
 CFUN_SMUL_ASSOC;REAL_ABS_NUM;COMPLEX_NORM_CX;COMPLEX_NORM_INV;REAL_INV_INV;GSYM CX_INV;GSYM CX_MUL;REAL_INV_MUL] THEN
 Pa.SUBGOAL_THEN "abs (sqrt (&(FACT n))) = sqrt (&(FACT n))" ASSUME_TAC THENL[ASM_SIMP_TAC[SQRT_LE_0; REAL_ABS_REFL;REAL_POS];ALL_TAC]
 THEN ASM_MESON_TAC[REAL_FIELD `&0< x ==> (x*y)* inv x = y`;SQRT_POS_LT;FACT_LT;REAL_OF_NUM_LT;CR_INDCR]);;
 
 let FOCK_HERM_VAC_ALT = prove
 (`!sm. is_sm6 sm ==> !n. (cr sm pow n) (vac sm) = Cx(sqrt(& (FACT n))) % fock sm n`,
  IMP_REWRITE_TAC[FOCK_HERM_VAC;COMPLEX_MUL_RINV;CFUN_SMUL_ASSOC;FACT_NZ;CX_INJ;REAL_OF_NUM_EQ;SQRT_EQ_0;CFUN_SMUL_LID]);;
  
 let INPROD_NORM = prove
 (`!inprod s x y. is_inner_space (s,inprod) /\ x IN s ==>
      (inprod x x = Cx(y) <=> cfun_norm inprod x = sqrt (y))`,
 REPEAT STRIP_TAC THEN EQ_TAC THENL[SIMP_TAC[REAL_OF_COMPLEX_CX;cfun_norm];ALL_TAC]
 THEN ASM_MESON_TAC[cfun_norm;REAL_OF_COMPLEX;INPROD_SELF_REAL;SQRT_INJ;GSYM (Pa.SPEC "inprod x x" REAL_OF_COMPLEX)]);;
 
 let FOCK_QST = prove
 (`!n sm. is_sm6 sm ==>  is_qst (fock sm n)`,
 REPEAT STRIP_TAC THEN 
 IMP_REWRITE_TAC[FOCK_HERM_VAC;is_qst;REWRITE_RULE [dh;danh;IN_ELIM_THM] CR_INDH;CFUN_SUBSPACE_SMUL;
 tem INNER_SPACE_IS_SUBSPACE;tem INPROD_NORM; tem CFUN_NORM_SMUL;NORM_UNORMAIZED_FOCK;SQRT_1;COMPLEX_NORM_INV;COMPLEX_NORM_CX]
 THEN Pa.SUBGOAL_THEN "abs (sqrt (&(FACT n))) = sqrt (&(FACT n))" ASSUME_TAC THENL[ASM_SIMP_TAC[SQRT_LE_0; REAL_ABS_REFL;REAL_POS];ALL_TAC]
 THEN ASM_SIMP_TAC[REAL_FIELD `&0< x ==> inv x* x = &1`;SQRT_POS_LT;FACT_LT;REAL_OF_NUM_LT]);;
 
 let FOCK_IN_S = prove
 (`!sm.
     is_sm6 sm ==>  !n. fock sm n IN sq_integrable`,
     ASM_MESON_TAC[FOCK_QST;is_qst]);;
	 
 let FOCK_EIGEN_PAIR = prove
 (`!sm n. is_sm6 sm ==>  is_eigen_pair_unbounded (dh2 sm)(phn sm) (fock sm n,Cx(&n))`,
REWRITE_TAC[is_eigen_pair_unbounded] THEN REPEAT STRIP_TAC THENL[
ASM_MESON_TAC[FOCK_HERM_VAC;UNNORMALIZED_FOCK;ULINCOP_SUBSPACE;CFUN_SUBSPACE_SMUL];
IMP_REWRITE_TAC[FOCK_HERM_VAC;UNNORMALIZED_FOCK;ULINCOP_SMUL;CFUN_SMUL_ASSOC] THEN CFUN_ARITH_TAC;
MP_TAC( Pa.SPECL ["n";"sm"] FOCK_QST) THEN ASM_SIMP_TAC[is_qst;tem INPROD_ZERO]
THEN CONV_TAC COMPLEX_FIELD]);;

let ANHH_FOCK = prove
(`!sm n. 
  is_sm6 sm ==> 
  (anh sm ) (fock sm (SUC n)) = Cx(sqrt(& (SUC n))) % fock sm n`,
  REPEAT STRIP_TAC THEN IMP_REWRITE_TAC[FOCK_HERM_VAC;REWRITE_RULE[is_unbounded_linear_cop]A_HERM_A_LINEAR;
  cop_pow;COP_MUL_THM;REWRITE_RULE[phn_sm;COP_MUL_THM] UNNORMALIZED_FOCK;REWRITE_RULE 
 [commutator;COP_SUB_THM;COP_MUL_THM;
 CFUN_ARITH `(x:cfun) - y = z <=> x =  z +y `;GSYM IMP_IMP] A_HERM_A_COMMUTATOR] THEN
 Pa.EXISTS_TAC " (cr sm pow n) (vac sm)" THEN
  ASM_SIMP_TAC[CR_INDCR;COP_ARITH `!op:cops. op ((op pow n) x) = (op pow (SUC n)) x`;CR_INDANH;
  GSYM (REWRITE_RULE[CFUN_SMUL_LID] (Pa.SPEC "Cx(&1)"CFUN_ADD_RDISTRIB));GSYM CX_ADD; REAL_OF_NUM_ADD;GSYM CX_MUL;
  GSYM CX_INV;SQRT_MUL;FACT;GSYM REAL_OF_NUM_MUL;REAL_INV_MUL; ARITH_RULE `1+m=SUC m`  ;CFUN_SMUL_ASSOC;
  REAL_FIELD `(inv x  * y) * (z:real) = y * (z/x)`;REAL_DIV_SQRT;REAL_POS] THEN CFUN_ARITH_TAC);;
  
let CREAT_FOCK = prove
(`!sm n. 
  is_sm6 sm ==> 
  cr sm (fock sm  n) = Cx(sqrt(& (SUC n))) % fock sm (SUC n)`,
  REPEAT STRIP_TAC THEN IMP_REWRITE_TAC[FOCK_HERM_VAC;REWRITE_RULE[is_unbounded_linear_cop]A_HERM_A_LINEAR]
  THEN Pa.EXISTS_TAC " (cr sm pow n) (vac sm)" THEN
  ASM_SIMP_TAC[CR_INDCR;FACT;GSYM CX_MUL;GSYM REAL_OF_NUM_MUL;SQRT_MUL;CFUN_SMUL_ASSOC;GSYM CX_INV;
  REAL_FIELD `&0 < x ==> x * inv x  * y = y`;LT_0;REAL_OF_NUM_LT;SQRT_POS_LT;REAL_INV_MUL;cop_pow;COP_MUL_THM]);;
  

  
 let VAC_OTHER_CR = prove
 (`! n sm1 sm2. is_sm6 sm1 ==> vac sm2  = vac sm1 ==> 
   (cr sm1 pow n) (vac sm2) = Cx(sqrt(& (FACT n))) % fock sm1 n `,
  SIMP_TAC[FOCK_HERM_VAC_ALT]);;
  
  let VAC_OTHER_CR_1 = REWRITE_RULE[ARITH `FACT (SUC 0) = 1`;SQRT_1;CFUN_SMUL_LID;cop_pow;COP_MUL_RID] (SPEC `SUC 0` VAC_OTHER_CR)
  
  
  



 
let  coherent = new_definition`coherent sm a =  
   Cx(exp (--((norm a) pow 2)/ (&2))) % 
     cfun_infsum (sq_integrable,r_inprod) (from 0) (\n. (a pow n / Cx(sqrt(& (FACT n)))) % fock sm n)` ;;  

 
let coherent_summable = new_definition`coherent_summable sm a <=>
 cfun_summable (sq_integrable,r_inprod)  (from 0) (\n. (a pow n / Cx(sqrt(& (FACT n)))) % fock sm n)`;;
 
let COHERENT_IN_SPC = prove
(`!sm a.
     coherent_summable sm a ==>  coherent sm a IN sq_integrable`,
	REWRITE_TAC[coherent;coherent_summable]
	THEN IMP_REWRITE_TAC[INNER_SPACE_IS_SUBSPACE;
	CFUN_SUBSPACE_SMUL;INFSUM_IN_SPC] THEN MESON_TAC[SQ_INTEGRABLE_INNER_SPACE]);; 	
	
let COHERENT_REWRITE = prove
 (`!sm a.
     is_sm6 sm /\ coherent_summable sm a ==>
   	(coherent sm a equv (Cx(exp (--((norm a) pow 2)/ (&2))) % 
	( vac sm+ 
	  cfun_infsum (sq_integrable,r_inprod) (from 0) (\n. (a pow (SUC n) / Cx(sqrt(& (FACT (SUC n))))) % fock sm (SUC n)))))r_inprod`,
	REWRITE_TAC[coherent;coherent_summable;ADD1]
	THEN REPEAT STRIP_TAC THEN
	 FIRST_ASSUM(MP_TAC o (SPEC `1`) o (MATCH_MP  ( REWRITE_RULE [GSYM IMP_IMP]CFUN_INFSUM_OFFSET)))
     THEN SIMP_TAC[SUB_REFL;CFUN_SUM_SING_NUMSEG;complex_pow;
	FACT;fock;SQRT_1;COMPLEX_DIV_1;CFUN_SMUL_LID;ARITH] 
	THEN Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
	ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	THEN  ASM_SIMP_TAC[] THEN DISCH_TAC THEN 
	FIRST_ASSUM(MP_TAC o (SPEC `1`) o (MATCH_MP  ( REWRITE_RULE [GSYM IMP_IMP]CFUN_SUMMABLE_OFFSET)))
    THEN ASM_SIMP_TAC[ARITH] THEN ONCE_REWRITE_TAC[ARITH_RULE `1=0+1`]
	THEN  DISCH_TAC THEN
	FIRST_ASSUM(MP_TAC o  (MATCH_MP  (REWRITE_RULE[ CFUN_SUMMABLE_REINDEX]CFUN_INFSUM_REINDEX)))
    THEN  ASM_SIMP_TAC[ARITH] THEN  DISCH_TAC THEN
	 IMP_REWRITE_TAC[EQUV_SMUL;CFUN_EQUV_LADD_ALT] THEN REPEAT STRIP_TAC 
	 THENL[MATCH_MP_TAC INPROD_EQUV_TRANSTIVE THEN 
	 Pa.EXISTS_TAC "sq_integrable"
    THEN 
	EXISTS_TAC `cfun_infsum (sq_integrable,r_inprod) (from (0 + 1))
	 (\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)` THEN REPEAT STRIP_TAC
	 THENL[SIMP_TAC[SQ_INTEGRABLE_INNER_SPACE];
    IMP_REWRITE_TAC[INFSUM_IN_SPC]
	THEN RULE_ASSUM_TAC(REWRITE_RULE[ARITH;GSYM CFUN_SUMMABLE_REINDEX])
	THEN ASM_REWRITE_TAC[];
	IMP_REWRITE_TAC[INFSUM_IN_SPC]
	THEN RULE_ASSUM_TAC(REWRITE_RULE[ARITH;GSYM CFUN_SUMMABLE_REINDEX])
	THEN ASM_REWRITE_TAC[];
	IMP_REWRITE_TAC[tem INNER_SPACE_SUB;ARITH;INFSUM_IN_SPC] THEN
	 ASM_MESON_TAC[SM_VAC_QST;is_qst];
	 MATCH_MP_TAC ( INPROD_EQUV_SYM_ALT) THEN 
	  Pa.EXISTS_TAC "sq_integrable" THEN REPEAT STRIP_TAC
	  THENL[SIMP_TAC[SQ_INTEGRABLE_INNER_SPACE];IMP_REWRITE_TAC[INFSUM_IN_SPC];
	  IMP_REWRITE_TAC[INFSUM_IN_SPC]
	  THEN RULE_ASSUM_TAC(REWRITE_RULE[ARITH;GSYM CFUN_SUMMABLE_REINDEX])
	  THEN ASM_REWRITE_TAC[ARITH]; ASM_REWRITE_TAC[ARITH]]; 
	  ASM_REWRITE_TAC[ARITH]];
	  IMP_REWRITE_TAC[INFSUM_IN_SPC]
	THEN RULE_ASSUM_TAC(REWRITE_RULE[ARITH;GSYM CFUN_SUMMABLE_REINDEX])
	THEN ASM_REWRITE_TAC[];ASM_MESON_TAC[SM_VAC_QST;is_qst];
	IMP_REWRITE_TAC[INFSUM_IN_SPC];ASM_MESON_TAC[REAL_EXP_NZ ;CX_INJ];
	IMP_REWRITE_TAC[tem INNER_SPACE_ADD;ARITH;INFSUM_IN_SPC]
	 THEN STRIP_TAC THENL[ASM_MESON_TAC[SM_VAC_QST;is_qst];ALL_TAC]
	THEN RULE_ASSUM_TAC(REWRITE_RULE[ARITH;GSYM CFUN_SUMMABLE_REINDEX])
	THEN ASM_REWRITE_TAC[]]);;

let COHERENT_REWRITE_ALT = prove
 (`!sm a.
     is_sm6 sm /\ coherent_summable sm a ==>
   	((Cx(exp (--((norm a) pow 2)/ (&2))) % 
	( vac sm+ cfun_infsum (sq_integrable,r_inprod) (from 0) (\n. (a pow (SUC n) / Cx(sqrt(& (FACT (SUC n))))) % fock sm (SUC n)))) 
	 equv coherent sm a)r_inprod`,	
	 REPEAT STRIP_TAC THEN MATCH_MP_TAC (INPROD_EQUV_SYM_ALT)
	 THEN Pa.EXISTS_TAC "sq_integrable" THEN REPEAT STRIP_TAC 
	 THEN ASM_SIMP_TAC[SQ_INTEGRABLE_INNER_SPACE;COHERENT_IN_SPC;COHERENT_REWRITE] THEN
	 IMP_REWRITE_TAC[tem INNER_SPACE_SMUL;tem INNER_SPACE_ADD] THEN 
	 CONJ_TAC THENL[ASM_MESON_TAC[SM_VAC_QST;is_qst];ALL_TAC] 
	 THEN IMP_REWRITE_TAC[INFSUM_IN_SPC] THEN 
	 RULE_ASSUM_TAC(REWRITE_RULE[coherent_summable]) THEN
	 FIRST_ASSUM(MP_TAC o (SPEC `1`) o (MATCH_MP  ( REWRITE_RULE [GSYM IMP_IMP]CFUN_SUMMABLE_OFFSET)))
     THEN Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
	ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	THEN ASM_SIMP_TAC [ARITH] THEN 
	ONCE_REWRITE_TAC [ARITH_RULE `1=0+1`] THEN  IMP_REWRITE_TAC[GSYM CFUN_SUMMABLE_REINDEX;ADD1]);;
	 

let VAC_COHERENT_0 = prove
 (`!sm.
     is_sm6 sm /\ coherent_summable sm (Cx(&0)) ==>
	   (coherent sm (Cx(&0)) equv vac sm) r_inprod`,
	  STRIP_TAC THEN DISCH_TAC THEN
	 FIRST_ASSUM(MP_TAC o  (MATCH_MP  COHERENT_REWRITE)) THEN
	 SIMP_TAC[COMPLEX_POW_ZERO;
	 NOT_SUC;COMPLEX_NORM_0;REAL_POW_ZERO;ARITH_RULE `~(2=0)`;
	 complex_div;real_div;REAL_NEG_0;REAL_MUL_LZERO;COMPLEX_MUL_LZERO;
	 CFUN_SMUL_LZERO;REAL_EXP_0;CFUN_SMUL_LID] THEN ASSUME_TAC (SPEC `from 0` (tem CFUN_INFSUM_0))
	 THEN  FIRST_ASSUM(MP_TAC o  (MATCH_MP  (CFUN_SUBST_RADD_ALT)))
	 THEN  DISCH_TAC THEN IMP_REWRITE_TAC[CFUN_ADD_RID] 
	 THEN REPEAT STRIP_TAC THENL[MATCH_MP_TAC INFSUM_IN_SPC
     THEN 	MATCH_MP_TAC CFUN_SUMS_SUMMABLE THEN ASM_MESON_TAC[tem CFUN_SERIES_0];
	 ASM_SIMP_TAC[tem INNER_SPACE_ZERO];ASM_MESON_TAC[SM_VAC_QST;is_qst];
     ASM_SIMP_TAC[COHERENT_IN_SPC]]);;	 
	  

let COHERENT_EIGEN_PAIR_CREAT = prove
 (`!sm a.
     is_sm6 sm /\  coherent_summable sm a /\ 
	 is_local_bounded1 (sq_integrable,r_inprod) (anh sm) 
     {x| ?n. x = cfun_sum ((from 0) INTER(0..n)) (\n. (a pow (SUC n) / Cx(sqrt(& (FACT (SUC n))))) % fock sm (SUC n)) 
	 - cfun_infsum (sq_integrable,r_inprod) (from 0) (\n. (a pow (SUC n) / Cx(sqrt(& (FACT (SUC n))))) % fock sm (SUC n))}	 
	 /\ (!m k. 
	 anh sm (cfun_infsum (sq_integrable,r_inprod) (from m) (\n. (a pow (n+k) / Cx(sqrt(& (FACT (n+k))))) % fock sm (n+k))) IN
	 danh sm)
	  ==> (anh sm (coherent sm a) equv a%(coherent sm a)) r_inprod`,
	  	 let complex_lemma = REWRITE_RULE[CX_INJ] 
	  (COMPLEX_FIELD `~(Cx(b) = Cx(&0)) ==> 
	   (a:complex) / (Cx(b) * c)  * Cx(b)= a /c`) in

	  REWRITE_TAC[are_equv] THEN  IMP_REWRITE_TAC[tem INPROD_SUB_RDIST; 
	  tem INPROD_SUB_LDIST; tem INPROD_RSMUL;tem INPROD_LSMUL;tem INNER_SPACE_SUB]
	  THEN REPEAT GEN_TAC THEN STRIP_TAC THEN 
	  FIRST_ASSUM(MP_TAC o SPEC `a:complex` o (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] COHERENT_REWRITE)))
	  THEN FIRST_ASSUM(ASSUME_TAC o (MATCH_MP HERM_A_A))
	  THEN FIRST_ASSUM(ASSUME_TAC o (MATCH_MP (tem ( HERM_ITSELF_UNBOUNDED))))
	  THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[GSYM th] THEN ASSUME_TAC th)
	  THEN FIRST_ASSUM(ASSUME_TAC o (MATCH_MP A_HERM_A))
	  THEN FIRST_ASSUM(ASSUME_TAC o (MATCH_MP (tem ( HERM_ITSELF_UNBOUNDED))))
	  THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[th] THEN ASSUME_TAC th)
	  THEN IMP_REWRITE_TAC[tem INPROD_EQUV_LREPLACE_ALT;tem INPROD_EQUV_RREPLACE_ALT]
	  THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[GSYM th] THEN MP_TAC th)
	  THEN POP_ASSUM(fun th -> ALL_TAC )
	  THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[th])
	  THEN DISCH_TAC THEN  POP_ASSUM(fun th -> IMP_REWRITE_TAC[th] THEN ASSUME_TAC th)
	 THEN IMP_REWRITE_TAC[tem INPROD_EQUV_RREPLACE_ALT] THEN
	 POP_ASSUM(fun th -> IMP_REWRITE_TAC[GSYM th] THEN ASSUME_TAC th)
	   THEN FIRST_ASSUM(ASSUME_TAC o (CONJUNCT1 ) o (MATCH_MP A_HERM_A_LINEAR))
	   THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_SMUL)))
	    THEN IMP_REWRITE_TAC[]
        THEN FIRST_ASSUM(MP_TAC o (SPEC `a:complex`) o (MATCH_MP ( REWRITE_RULE [GSYM IMP_IMP] COHERENT_REWRITE_ALT)))
		THEN ASM_SIMP_TAC[] THEN DISCH_TAC THEN
		FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_LREPLACE_ALT))) THEN
		POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th])
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_RREPLACE_ALT))) THEN
		POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th])
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_ADD)))
	    THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
		THEN IMP_REWRITE_TAC[tem INPROD_RSMUL;tem INPROD_LSMUL]
	    THEN REPEAT STRIP_TAC THENL[
		IMP_REWRITE_TAC[tem INPROD_ADD_LDIST; tem INPROD_ADD_RDIST]
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP A_VAC))
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_LREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_RREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
	    THEN IMP_REWRITE_TAC[tem INPROD_LZERO;tem INPROD_RZERO;COMPLEX_ADD_LID]
		THEN Pa.SUBGOAL_THEN " cfun_summable (sq_integrable,r_inprod) (from 0) 
		(\\n. (a pow SUC n / Cx (sqrt (&(FACT (SUC n))))) % fock sm (SUC n)) "
		ASSUME_TAC THENL[REWRITE_TAC[SPEC_V ("k","1")CFUN_SUMMABLE_REINDEX;ADD1;ADD]
		THEN IMP_REWRITE_TAC[ARITH_RULE  `0 < 1`;SPEC_V ("n","1") (SPEC_V ("m","0") CFUN_SUMMABLE_OFFSET)]
		THEN  ASM_MESON_TAC[coherent_summable;FOCK_IN_S;tem INNER_SPACE_SMUL];ALL_TAC]
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP]  CFUN_INFSUM_SLINEAR_ALT))) THEN
	    POP_ASSUM(fun th -> FIRST_ASSUM (ASSUME_TAC o  (MATCH_MP th)))
		THEN POP_ASSUM(fun th -> FIRST_ASSUM (MP_TAC o  (MATCH_MP th)))
		THEN REWRITE_TAC[danh;IN_ELIM_THM;BETA_THM] THEN IMP_REWRITE_TAC[]
		THEN ASM_SIMP_TAC[danh; IN_ELIM_THM;ANHH_FOCK;FOCK_IN_S;tem INNER_SPACE_SMUL;INFSUM_IN_SPC]
		THEN SUBGOAL_THEN `anh sm
  (cfun_infsum (sq_integrable,r_inprod) (from 0)
  (\n. (a pow SUC n / Cx (sqrt (&(FACT (SUC n))))) % fock sm (SUC n))) IN
  sq_integrable` ASSUME_TAC THENL[
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;ADD1;IN_ELIM_THM]
		;ALL_TAC] THEN ASM_SIMP_TAC[] THEN 
         DISCH_TAC THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_RREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (tem INPROD_EQUV_LREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th) THEN 
		
		
	   ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;
	GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;
    CX_MUL;GSYM CFUN_INFSUM_SMUL;complex_lemma;LT_0;REAL_OF_NUM_LT;
    SQRT_POS_LT;REAL_POS_NZ] THEN
	 
	 SUBGOAL_THEN `cfun_summable (sq_integrable,r_inprod) (from 0)
      (\n. (a pow n / Cx (sqrt (&(FACT (n))))) % fock sm (n))`  ASSUME_TAC
	  THENL[ASM_MESON_TAC[coherent_summable];ALL_TAC]
	   THEN FIRST_ASSUM(ASSUME_TAC o SPEC `a:complex` o (MATCH_MP (CFUN_INFSUM_SMUL)))
	   THEN REWRITE_TAC[GSYM COMPLEX_MUL_ASSOC;complex_div]
	THEN FIRST_ASSUM(ASSUME_TAC o  (REWRITE_RULE[CFUN_SMUL_ASSOC;GSYM COMPLEX_MUL_ASSOC;complex_div])
	o (MATCH_MP (tem INPROD_EQUV_RREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
		THEN FIRST_ASSUM(ASSUME_TAC o  (REWRITE_RULE[CFUN_SMUL_ASSOC;GSYM COMPLEX_MUL_ASSOC;complex_div])
		o(MATCH_MP (tem INPROD_EQUV_LREPLACE_ALT)))
		THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
		THEN IMP_REWRITE_TAC[coherent;GSYM COMPLEX_MUL_ASSOC;complex_div;tem INPROD_RSMUL;tem INPROD_LSMUL]
		
		THEN REPEAT STRIP_TAC THENL[CONV_TAC COMPLEX_FIELD;
		ASM_SIMP_TAC[GSYM complex_div;GSYM coherent_summable;INFSUM_IN_SPC]
		;ASM_SIMP_TAC[tem INNER_SPACE_SMUL;GSYM complex_div;GSYM coherent_summable;INFSUM_IN_SPC]
		;ASM_SIMP_TAC[tem INNER_SPACE_SMUL;GSYM complex_div;GSYM coherent_summable;INFSUM_IN_SPC]
		;REWRITE_TAC[GSYM complex_div]
		THEN RULE_ASSUM_TAC(REWRITE_RULE[cfun_summable;coherent_summable])
		THEN FIRST_ASSUM(X_CHOOSE_TAC `l:real->real^2`)
		THEN IMP_REWRITE_TAC[INFSUM_IN_SPC;CFUN_SUMS_SUMMABLE; GSYM CFUN_SMUL_ASSOC]
		THEN EXISTS_TAC `a%l:real->real^2` THEN ASM_SIMP_TAC[CFUN_SERIES_SMUL] 
		;ASM_SIMP_TAC[tem INNER_SPACE_ZERO];		
		ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		;IMP_REWRITE_TAC[tem INNER_SPACE_ADD] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		THEN 		
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC]];
		ASM_SIMP_TAC[COHERENT_IN_SPC];
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		THEN 		
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC];
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		THEN 		
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC];
		ASM_SIMP_TAC[danh;IN_ELIM_THM;VAC_QST_PROP;REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)];
		REWRITE_TAC[danh;IN_ELIM_THM] THEN CONJ_TAC THENL[ALL_TAC;
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC]] THEN 
		IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[];
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
	    THEN IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[];
		REWRITE_TAC[IN_ELIM_THM;danh] THEN REPEAT STRIP_TAC THENL[
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
	    THEN IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[];ALL_TAC]
		THEN POP_ASSUM (fun th -> ALL_TAC) THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
	    THEN IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_ADD] THEN REPEAT STRIP_TAC
		THENL[ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)];
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC];
		ASM_SIMP_TAC[VAC_QST_PROP];
		IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[]];
		ASM_SIMP_TAC[GSYM DOMAINS_EQ;danh;IN_ELIM_THM;COHERENT_IN_SPC]
		THEN IMP_REWRITE_TAC[tem INNER_SPACE_SMUL;coherent] THEN
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN 
		SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM] THEN 
		ASM_SIMP_TAC[INFSUM_IN_SPC;GSYM coherent_summable];
		REWRITE_TAC[IN_ELIM_THM;danh] THEN REPEAT STRIP_TAC THENL[
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
	    THEN IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[];ALL_TAC]
		THEN POP_ASSUM (fun th -> ALL_TAC) THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
	    THEN IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN
		POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th) THEN 
		IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL] THEN REPEAT STRIP_TAC
		THENL[ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)];
		FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC];
		ASM_SIMP_TAC[VAC_QST_PROP];
		IMP_REWRITE_TAC[INFSUM_IN_SPC;ADD1] THEN 
		Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	    THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from (0+1))
		(\\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)" ASSUME_TAC THENL[
		REWRITE_TAC[ARITH] THEN MATCH_MP_TAC (GEN_ALL CFUN_SUMMABLE_OFFSET) THEN 
		Pa.EXISTS_TAC "0" THEN ASM_SIMP_TAC[ARITH;GSYM coherent_summable];ALL_TAC] 
		THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMMABLE_REINDEX]) THEN ASM_REWRITE_TAC[]];
		ASM_SIMP_TAC[GSYM DOMAINS_EQ;danh;IN_ELIM_THM] THEN 
		IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL]
		THEN POP_ASSUM (fun th -> ALL_TAC) THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] THEN ASSUME_TAC th)
	    THEN IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL]		
		THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC]
		THEN DISCH_TAC THEN MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) 
		 THEN Pa.EXISTS_TAC "anh sm (vac sm) IN sq_integrable"
		 THEN ASM_SIMP_TAC[COP_MUL_THM; (GSYM (REWRITE_RULE[IN_ELIM_THM;EXTENSION] danh));
		 DOMAINS_EQ;dcr;IN_ELIM_THM;REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)] THEN
		 ASM_SIMP_TAC[GSYM phn_sm;GSYM COP_MUL_THM;
		 REWRITE_RULE[is_eigen_pair_unbounded]VAC_ZERO_PHOTON;N_LINEAR;CFUN_SMUL_LZERO;tem INNER_SPACE_ZERO];
		FIRST_ASSUM(ASSUME_TAC o (CONJUNCT2) o (MATCH_MP A_HERM_A_LINEAR))
	   THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_SMUL)))
	    THEN IMP_REWRITE_TAC[]
		THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_ADD)))
	    THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] )
		THEN ASM_SIMP_TAC[GSYM DOMAINS_EQ;danh;IN_ELIM_THM]
		THEN POP_ASSUM (fun th -> ALL_TAC) THEN POP_ASSUM (fun th -> ALL_TAC) THEN 
    	FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_ADD)))
	    THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[ th] ) THEN
		ASM_SIMP_TAC[GSYM DOMAINS_EQ;danh;IN_ELIM_THM] THEN 
		IMP_REWRITE_TAC[tem INNER_SPACE_ADD;tem INNER_SPACE_SMUL]
	    THEN ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)]
		THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`])) THEN SIMP_TAC[danh;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC] 
		THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`1`]))  
		THEN FIRST_ASSUM(fun th -> REWRITE_TAC[(MATCH_MP DOMAINS_EQ) th] )
		THEN ASM_REWRITE_TAC[DOMAINS_EQ;dcr;COMPLEX_MUL_LID;IN_ELIM_THM]
		THEN ASM_SIMP_TAC[FACT;complex_pow;CFUN_SMUL_ASSOC;complex_div;
		GSYM REAL_OF_NUM_MUL;SQRT_MUL;REAL_POS;CX_MUL;GSYM ADD1;COMPLEX_MUL_ASSOC]
		 THEN ASM_SIMP_TAC[GSYM phn_sm;GSYM COP_MUL_THM;
		 REWRITE_RULE[is_eigen_pair_unbounded]VAC_ZERO_PHOTON;N_LINEAR;CFUN_SMUL_LZERO;tem INNER_SPACE_ZERO]
		 THEN  DISCH_TAC THEN MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) 
		 THEN Pa.EXISTS_TAC "anh sm (vac sm) IN sq_integrable"
		 THEN ASM_SIMP_TAC[COP_MUL_THM; (GSYM (REWRITE_RULE[IN_ELIM_THM;EXTENSION] danh));
		 DOMAINS_EQ;dcr;IN_ELIM_THM] THEN
		 ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)] THEN
		 ASM_SIMP_TAC[GSYM phn_sm;GSYM COP_MUL_THM;
		 REWRITE_RULE[is_eigen_pair_unbounded]VAC_ZERO_PHOTON;N_LINEAR;CFUN_SMUL_LZERO;tem INNER_SPACE_ZERO];
		 ASM_SIMP_TAC[danh;IN_ELIM_THM;COHERENT_IN_SPC] THEN
		 IMP_REWRITE_TAC[coherent;danh;IN_ELIM_THM;tem INNER_SPACE_SMUL;INFSUM_IN_SPC;COHERENT_IN_SPC;GSYM coherent_summable]
		 THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM];
		 ASM_SIMP_TAC[GSYM DOMAINS_EQ;danh;IN_ELIM_THM;coherent] THEN
		 IMP_REWRITE_TAC[danh;IN_ELIM_THM;tem INNER_SPACE_SMUL;INFSUM_IN_SPC;COHERENT_IN_SPC;GSYM coherent_summable]
		 THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM];
		 MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) 
		 THEN Pa.EXISTS_TAC "(coherent sm a) IN sq_integrable"
		  THEN ASM_SIMP_TAC[COP_MUL_THM; (GSYM (REWRITE_RULE[IN_ELIM_THM;EXTENSION] dcr));
		 GSYM DOMAINS_EQ] THEN  ASM_SIMP_TAC[danh;IN_ELIM_THM;COHERENT_IN_SPC] THEN
		 IMP_REWRITE_TAC[coherent;danh;IN_ELIM_THM;tem INNER_SPACE_SMUL;INFSUM_IN_SPC;COHERENT_IN_SPC;GSYM coherent_summable]
		 THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM];  
		 MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) 
		 THEN Pa.EXISTS_TAC "anh sm (coherent sm a) IN sq_integrable"
		  THEN ASM_SIMP_TAC[COP_MUL_THM; (GSYM (REWRITE_RULE[IN_ELIM_THM;EXTENSION] dcr));
		 GSYM DOMAINS_EQ] THEN ASM_SIMP_TAC[danh;IN_ELIM_THM;COHERENT_IN_SPC] THEN
		 IMP_REWRITE_TAC[coherent;danh;IN_ELIM_THM;tem INNER_SPACE_SMUL;INFSUM_IN_SPC;COHERENT_IN_SPC;GSYM coherent_summable]
		 THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM];
		 FIRST_ASSUM(ASSUME_TAC o (CONJUNCT1 ) o (MATCH_MP A_HERM_A_LINEAR))
	   THEN FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP] ULINCOP_SMUL)))
	    THEN IMP_REWRITE_TAC[coherent;danh;IN_ELIM_THM;tem INNER_SPACE_SMUL;INFSUM_IN_SPC;COHERENT_IN_SPC;GSYM coherent_summable]
		 THEN FIRST_ASSUM(MP_TAC o (SPECL[`0`;`0`])) THEN SIMP_TAC[ARITH_RULE `n+0=n`;COMPLEX_MUL_LID;danh;IN_ELIM_THM];
		 ASM_SIMP_TAC[COHERENT_IN_SPC;tem INNER_SPACE_SMUL]]
		 THEN DISCH_TAC THEN MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) 
		 THEN Pa.EXISTS_TAC "anh sm (vac sm) IN sq_integrable"
		 THEN ASM_SIMP_TAC[COP_MUL_THM; (GSYM (REWRITE_RULE[IN_ELIM_THM;EXTENSION] danh));
		 DOMAINS_EQ;dcr;IN_ELIM_THM] THEN
		 ASM_SIMP_TAC[REWRITE_RULE[cop_pow;I_THM;IN_ELIM_THM;danh](SPEC_V ("m","0") CR_INDANH)] THEN
		 ASM_SIMP_TAC[GSYM phn_sm;GSYM COP_MUL_THM;
		 REWRITE_RULE[is_eigen_pair_unbounded]VAC_ZERO_PHOTON;N_LINEAR;CFUN_SMUL_LZERO;tem INNER_SPACE_ZERO] );;


   
		 
let EXP_N_FOCK = prove
(`!a m sm. is_sm6 sm ==> !n. ((a% (phn sm)) pow n) (fock sm m) = (a pow n * Cx(&m) pow n) %fock sm m`,
	  REPEAT GEN_TAC THEN STRIP_TAC THEN INDUCT_TAC THENL[
	  REWRITE_TAC[cop_pow;complex_pow;COMPLEX_MUL_RID;CFUN_SMUL_LID;I_THM];ALL_TAC]
	  THEN ASM_SIMP_TAC[cop_pow;COP_MUL_THM] THEN IMP_REWRITE_TAC[ULINCOP_SMUL]
	  THEN Pa.EXISTS_TAC "dh2 sm" THEN ASM_SIMP_TAC[N_LINEAR;SMUL_ULINCOP]
	  THEN ASM_SIMP_TAC[N_LINEAR;REWRITE_RULE[is_eigen_pair_unbounded]FOCK_EIGEN_PAIR;
	  COP_SMUL_THM;CFUN_SMUL_ASSOC;GSYM COMPLEX_POW_MUL ] THEN REWRITE_TAC[complex_pow;COMPLEX_MUL_SYM]);;

let FOCK_PHASE_SHIFT = prove 
 (`!sm m w.
     is_sm6 sm ==>
     cfun_sums (sq_integrable,r_inprod)  (\n. Cx (inv(&(FACT n))) % (((ii * Cx w) % (phn sm)) pow n) (fock sm m))
	 ((cexp (ii * Cx w) pow m) % fock sm m) (from 0)`, 
	REPEAT STRIP_TAC THEN
	ASM_SIMP_TAC[CFUN_SMUL;EXP_N_FOCK;GSYM COMPLEX_POW_MUL;GSYM CEXP_N;COMPLEX_MUL_SYM] 
    THEN MATCH_MP_TAC (REWRITE_RULE[CFUN_SMUL;GSYM COMPLEX_MUL_ASSOC]COP_EXP_SCALAR)
    THEN ASM_SIMP_TAC[ETA_AX;FOCK_IN_S]);;
  
let  FOCK_PHASE_SHIFTER =   prove 
 (`!sm m w.
     is_sm6 sm ==>
       (cfun_infsum (sq_integrable,r_inprod)  (from 0) (\n. Cx (inv(&(FACT n))) % (((ii * Cx w) % (phn sm)) pow n) (fock sm m))
	   equv ((cexp (ii * Cx w) pow m) % (fock sm m)))r_inprod`,
    IMP_REWRITE_TAC[CFUN_INFSUM_UNIQUE;FOCK_PHASE_SHIFT]);;


let  FOCK_PHASE_SHIFTER_ALT =   prove 
 (`!sm m w a.
     is_sm6 sm ==>
       ((a m)% cfun_infsum (sq_integrable,r_inprod)  (from 0) (\n. Cx (inv(&(FACT n))) % (((ii * Cx w) % (phn sm)) pow n) (fock sm m))
	   equv (a m)%((cexp (ii * Cx w) pow m) % (fock sm m)))r_inprod`, 
	   IMP_REWRITE_TAC[FOCK_PHASE_SHIFTER;EQUV_SMUL_ALT;tem INNER_SPACE_SMUL;FOCK_IN_S;INFSUM_IN_SPC]
	THEN ASM_MESON_TAC[cfun_summable;FOCK_PHASE_SHIFT]);;

let CEXP_II_PI = prove(`cexp (ii * Cx(pi)) = --Cx(&1)`,
    REWRITE_TAC[CEXP_EULER;GSYM CX_COS;GSYM CX_SIN;COS_PI;
    SIN_PI;COMPLEX_ADD_RID;COMPLEX_MUL_RZERO;CX_NEG]);;  


let cop_exp= new_definition`cop_exp op = (\x. cfun_infsum (sq_integrable,r_inprod) (from 0)
	  (\n. Cx (inv(&(FACT n))) % (op pow n ) x))`;;
	  
let exp_summable = new_definition `exp_summable op x <=> cfun_summable (sq_integrable,r_inprod) (from 0)
	  (\n. Cx (inv(&(FACT n))) % (op pow n ) x)`;;

let flip_gate =  new_definition `flip_gate sm = cop_exp ((ii * Cx pi) % (phn sm))`;;


let MIRROR_COHERENT = prove
 (`!sm a. is_sm6 sm /\
      is_local_bounded1 (sq_integrable,r_inprod)  (flip_gate sm) 
	  {x| ?n. x = cfun_sum ((from 0) INTER(0..n)) (\n. (a pow ( n) / Cx(sqrt(& (FACT ( n))))) % fock sm ( n)) 
	 - cfun_infsum (sq_integrable,r_inprod) (from 0) (\n. (a pow ( n) / Cx(sqrt(& (FACT ( n))))) % fock sm ( n))}	 
	 /\
	   is_unbounded_linear_cop {x|x IN sq_integrable /\ exp_summable ((ii * Cx pi) % (phn sm)) x}
	   (flip_gate sm)
	   /\ (!l. cfun_sums (sq_integrable,r_inprod) 
         (\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n) l (from 0) ==> 
		 l IN {x|x IN sq_integrable /\ exp_summable ((ii * Cx pi) % (phn sm)) x} )
	  /\ coherent_summable sm (--a) /\ coherent_summable sm a
	 ==> (( flip_gate sm) (coherent sm a) equv
	  (coherent sm (--a)))r_inprod`, 
	  REWRITE_TAC[coherent;coherent_summable;flip_gate;exp_summable;cop_exp]
	   THEN REPEAT STRIP_TAC  THEN
	  FIRST_ASSUM(ASSUME_TAC o  (MATCH_MP (REWRITE_RULE [GSYM IMP_IMP]  CFUN_INFSUM_SLINEAR_ALT)))
	  THEN POP_ASSUM (fun th -> FIRST_ASSUM (ASSUME_TAC o (MATCH_MP th)))
	  THEN POP_ASSUM (fun th -> FIRST_ASSUM (ASSUME_TAC o (MATCH_MP th)))
	  
	 THEN FIRST_ASSUM(ASSUME_TAC o   (REWRITE_RULE[BETA_THM;IN_ELIM_THM] )o(MATCH_MP (REWRITE_RULE [GSYM IMP_IMP]  ULINCOP_SMUL)))
	 	THEN IMP_REWRITE_TAC[] THEN IMP_REWRITE_TAC[NORM_NEG;EQUV_SMUL_ALT]
	   THEN POP_ASSUM (fun th -> ALL_TAC)  THEN POP_ASSUM MP_TAC  
	  THEN ASM_SIMP_TAC[IN_ELIM_THM;INFSUM_IN_SPC] THEN IMP_REWRITE_TAC[tem INNER_SPACE_SMUL;FOCK_IN_S]
	  THEN Pa.SUBGOAL_THEN "!x. (a pow x / Cx (sqrt (&(FACT x)))) % fock sm x IN sq_integrable"
		ASSUME_TAC THENL[ASM_MESON_TAC[FOCK_IN_S;SQ_INTEGRABLE_INNER_SPACE;INNER_SPACE_SMUL];ALL_TAC]
	 
	  THEN FIRST_ASSUM(MP_TAC o  GEN_ALL o (SPECL [`a:complex`;`fock sm n`])o (REWRITE_RULE[BETA_THM;IN_ELIM_THM] )o(MATCH_MP (REWRITE_RULE [GSYM IMP_IMP]  ULINCOP_SMUL)))
	   THEN Pa.SUBGOAL_THEN "(!n. cfun_summable (sq_integrable,r_inprod) (from 0)
	   (\\n'. Cx (inv (&(FACT n'))) %  ((ii * Cx pi) % phn sm pow n') (fock sm n)))" ASSUME_TAC
	   THENL[REPEAT STRIP_TAC THEN REWRITE_TAC[cfun_summable]
	  THEN Pa.EXISTS_TAC "(cexp (ii * Cx pi) pow n) % fock sm n"
	  THEN ASM_SIMP_TAC[FOCK_PHASE_SHIFT];ALL_TAC] THEN
	   ASM_SIMP_TAC[FOCK_IN_S] THEN DISCH_TAC THEN
	   
	   FIRST_ASSUM(ASSUME_TAC  o GEN `m:num` o (REWRITE_RULE[BETA_THM])o (SPECL [`m:num`;`pi`;`\n. (a pow n / Cx (sqrt (&(FACT n))))`]) o (MATCH_MP FOCK_PHASE_SHIFTER_ALT))
	  THEN  FIRST_ASSUM(MP_TAC   o (MATCH_MP (REWRITE_RULE[GSYM IMP_IMP;BETA_THM] 
	   (Pa.SPECL["\\m. cfun_infsum (sq_integrable,r_inprod) (from 0)
           (\\n. Cx (inv (&(FACT n))) %
                ((ii * Cx pi) % phn sm pow n) (fock sm m)) ";"\\m. ((cexp (ii * Cx pi) pow m) % fock sm m)"]EQUV_UNDER_INFSUM_SMUL))))
	  
	  THEN Pa.SUBGOAL_THEN "cfun_summable (sq_integrable,r_inprod) (from 0)
  (\\m. (a pow m / Cx (sqrt (&(FACT m)))) %
       cfun_infsum (sq_integrable,r_inprod) (from 0)
       (\\n. Cx (inv (&(FACT n))) % ((ii * Cx pi) % phn sm pow n) (fock sm m)))"
	    ASSUME_TAC THENL[
		RULE_ASSUM_TAC(GSYM )
	 THEN ASM_REWRITE_TAC[]
     THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMS_INFSUM] ) THEN
	 POP_ASSUM (fun th -> ALL_TAC) THEN  POP_ASSUM (fun th ->ALL_TAC) THEN 
   FIRST_ASSUM(ASSUME_TAC o (MATCH_MP (REWRITE_RULE[GSYM IMP_IMP] CFUN_SERIES_SLINEAR)))
    	THEN POP_ASSUM(fun th -> FIRST_ASSUM(ASSUME_TAC o (MATCH_MP th)))
	THEN REWRITE_TAC[cfun_summable]
	 THEN EXISTS_TAC `  cfun_infsum (sq_integrable,r_inprod) (from 0)
	   (\n. Cx (inv (&(FACT n))) % ((ii * Cx pi) % phn sm pow n) 
	   (cfun_infsum (sq_integrable,r_inprod) (from 0)
          (\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)))`
	THEN POP_ASSUM(fun th -> MATCH_MP_TAC (REWRITE_RULE [IMP_IMP;BETA_THM] th))
	THEN RULE_ASSUM_TAC(GSYM o (REWRITE_RULE[ CFUN_SUMS_INFSUM;IN_ELIM_THM]) )
	THEN ASM_SIMP_TAC[CFUN_SUMS_INFSUM;IN_ELIM_THM] THEN REPEAT STRIP_TAC
	THENL[IMP_REWRITE_TAC[INFSUM_IN_SPC];MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) THEN
	 Pa.EXISTS_TAC " ((a pow m / Cx (sqrt (&(FACT m)))) % fock sm m) IN sq_integrable"
	 THEN SUBGOAL_THEN `(((a pow m / Cx (sqrt (&(FACT m)))) % fock sm m)  IN sq_integrable /\
 cfun_summable (sq_integrable,r_inprod) (from 0)
 (\n'. Cx (inv (&(FACT n'))) %
       ((ii * Cx pi) % phn sm pow n')
       ((a pow m / Cx (sqrt (&(FACT m)))) % fock sm m))) <=> (
	   ((a pow m / Cx (sqrt (&(FACT m)))) % fock sm m)  IN
	   {x|x IN sq_integrable /\ cfun_summable (sq_integrable,r_inprod) (from 0)
	  (\n. Cx (inv(&(FACT n))) % (((ii * Cx pi) % (phn sm)) pow n ) x)})` ASSUME_TAC
	  THENL[REWRITE_TAC[IN_ELIM_THM];ALL_TAC] 
	THEN ASM_REWRITE_TAC[] THEN MATCH_MP_TAC CFUN_SUBSPACE_SMUL
	THEN ASM_SIMP_TAC[IN_ELIM_THM;FOCK_IN_S;cfun_summable;FOCK_PHASE_SHIFT]
	THEN REWRITE_TAC[GSYM cfun_summable] THEN IMP_REWRITE_TAC[ ULINCOP_SUBSPACE]]
		;ALL_TAC]
	  
	  THEN Pa.SUBGOAL_THEN " cfun_summable (sq_integrable,r_inprod) (from 0)
      (\m. (a pow m / Cx (sqrt (&(FACT m)))) %
           ((--Cx (&1) pow m) % fock sm m))" ASSUME_TAC THENL [REWRITE_TAC[CEXP_II_PI;CFUN_SMUL_ASSOC;COMPLEX_POW_MUL;COMPLEX_MUL_RID;COMPLEX_POW_ONE;complex_div;GSYM COMPLEX_MUL_ASSOC;COMPLEX_MUL_AC]
    THEN REWRITE_TAC[GSYM COMPLEX_POW_MUL;COMPLEX_FIELD `a * --Cx(&1) = --a`]
	THEN ONCE_REWRITE_TAC[COMPLEX_MUL_SYM]
	THEN  ASM_REWRITE_TAC[GSYM complex_div]
	;ALL_TAC] THEN 
	ASM_SIMP_TAC[] THEN 
    REPEAT STRIP_TAC THENL[
		IMP_REWRITE_TAC[INFSUM_IN_SPC]
	  THEN RULE_ASSUM_TAC(REWRITE_RULE[IN_ELIM_THM;GSYM CFUN_SUMS_INFSUM])
	  THEN REPEAT (FIRST_X_ASSUM(fun th -> FIRST_ASSUM (fun th1-> ASSUME_TAC ( MATCH_MP th1 th))))
	  THEN ASM_REWRITE_TAC[GSYM CFUN_SUMS_INFSUM]
	  THEN  IMP_REWRITE_TAC[INFSUM_IN_SPC];
	MATCH_MP_TAC INPROD_EQUV_TRANSTIVE
   THEN Pa.EXISTS_TAC "sq_integrable" THEN Pa.EXISTS_TAC "cfun_infsum (sq_integrable,r_inprod) (from 0)
       (\\m.  (a pow m / Cx (sqrt (&(FACT m)))) %
                ((cexp (ii * Cx pi) pow m) % fock sm m))"
	THEN REPEAT STRIP_TAC THENL[
	REWRITE_TAC[SQ_INTEGRABLE_INNER_SPACE];
	IMP_REWRITE_TAC[INFSUM_IN_SPC]
	  THEN RULE_ASSUM_TAC(REWRITE_RULE[IN_ELIM_THM;GSYM CFUN_SUMS_INFSUM])
	  THEN REPEAT (FIRST_X_ASSUM(fun th -> FIRST_ASSUM (fun th1-> ASSUME_TAC ( MATCH_MP th1 th))))
	  THEN ASM_REWRITE_TAC[GSYM CFUN_SUMS_INFSUM]
	  THEN  IMP_REWRITE_TAC[INFSUM_IN_SPC];
	  IMP_REWRITE_TAC[INFSUM_IN_SPC]
	THEN REWRITE_TAC[CEXP_II_PI;CFUN_SMUL_ASSOC;COMPLEX_POW_MUL;COMPLEX_MUL_RID;COMPLEX_POW_ONE;complex_div;GSYM COMPLEX_MUL_ASSOC;COMPLEX_MUL_AC]
    THEN REWRITE_TAC[GSYM COMPLEX_POW_MUL;COMPLEX_FIELD `a * --Cx(&1) = --a`]
	THEN ONCE_REWRITE_TAC[COMPLEX_MUL_SYM]
	THEN  ASM_REWRITE_TAC[GSYM complex_div];
	IMP_REWRITE_TAC[INFSUM_IN_SPC];
	MATCH_MP_TAC INPROD_EQUV_TRANSTIVE	
    THEN Pa.EXISTS_TAC "sq_integrable" THEN 
    Pa.EXISTS_TAC "cfun_infsum (sq_integrable,r_inprod) (from 0) (\\n. (a pow n / Cx (sqrt (&(FACT n)))) %cfun_infsum (sq_integrable,r_inprod) (from 0)(\\n'. Cx (inv (&(FACT n'))) %((ii * Cx pi) % phn sm pow n') (fock sm n)))"	
	THEN POP_ASSUM(fun th -> IMP_REWRITE_TAC[th])
	THEN REPEAT STRIP_TAC THENL [REWRITE_TAC[SQ_INTEGRABLE_INNER_SPACE];
	IMP_REWRITE_TAC[INFSUM_IN_SPC]
	  THEN RULE_ASSUM_TAC(REWRITE_RULE[IN_ELIM_THM;GSYM CFUN_SUMS_INFSUM])
	  THEN REPEAT (FIRST_X_ASSUM(fun th -> FIRST_ASSUM (fun th1-> ASSUME_TAC ( MATCH_MP th1 th))))
	  THEN ASM_REWRITE_TAC[GSYM CFUN_SUMS_INFSUM]
	  THEN  IMP_REWRITE_TAC[INFSUM_IN_SPC];
	  IMP_REWRITE_TAC[INFSUM_IN_SPC] THEN
	  RULE_ASSUM_TAC(GSYM )
	 THEN ASM_REWRITE_TAC[]
     THEN RULE_ASSUM_TAC(REWRITE_RULE[GSYM CFUN_SUMS_INFSUM] ) THEN
	 POP_ASSUM (fun th -> ALL_TAC) THEN  POP_ASSUM (fun th ->ALL_TAC) THEN 
   FIRST_ASSUM(ASSUME_TAC o (MATCH_MP (REWRITE_RULE[GSYM IMP_IMP] CFUN_SERIES_SLINEAR)))
    	THEN POP_ASSUM(fun th -> FIRST_ASSUM(ASSUME_TAC o (MATCH_MP th)))
	THEN REWRITE_TAC[cfun_summable]
	 THEN EXISTS_TAC `  cfun_infsum (sq_integrable,r_inprod) (from 0)
	   (\n. Cx (inv (&(FACT n))) % ((ii * Cx pi) % phn sm pow n) 
	   (cfun_infsum (sq_integrable,r_inprod) (from 0)
          (\n. (a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)))`
	THEN POP_ASSUM(fun th -> MATCH_MP_TAC (REWRITE_RULE [IMP_IMP;BETA_THM] th))
	THEN RULE_ASSUM_TAC(GSYM o (REWRITE_RULE[ CFUN_SUMS_INFSUM;IN_ELIM_THM]) )
	THEN ASM_SIMP_TAC[CFUN_SUMS_INFSUM;IN_ELIM_THM] THEN REPEAT STRIP_TAC
	THENL[IMP_REWRITE_TAC[INFSUM_IN_SPC];MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) THEN
	 Pa.EXISTS_TAC " ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n) IN sq_integrable"
	 THEN SUBGOAL_THEN `(((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)  IN sq_integrable /\
 cfun_summable (sq_integrable,r_inprod) (from 0)
 (\n'. Cx (inv (&(FACT n'))) %
       ((ii * Cx pi) % phn sm pow n')
       ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n))) <=> (
	   ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)  IN
	   {x|x IN sq_integrable /\ cfun_summable (sq_integrable,r_inprod) (from 0)
	  (\n. Cx (inv(&(FACT n))) % (((ii * Cx pi) % (phn sm)) pow n ) x)})` ASSUME_TAC
	  THENL[REWRITE_TAC[IN_ELIM_THM];ALL_TAC] 
	THEN ASM_REWRITE_TAC[] THEN MATCH_MP_TAC CFUN_SUBSPACE_SMUL
	THEN ASM_SIMP_TAC[IN_ELIM_THM;FOCK_IN_S;cfun_summable;FOCK_PHASE_SHIFT]
	THEN REWRITE_TAC[GSYM cfun_summable] THEN IMP_REWRITE_TAC[ ULINCOP_SUBSPACE]];
	IMP_REWRITE_TAC[INFSUM_IN_SPC]
	THEN REWRITE_TAC[CEXP_II_PI;CFUN_SMUL_ASSOC;COMPLEX_POW_MUL;COMPLEX_MUL_RID;COMPLEX_POW_ONE;complex_div;GSYM COMPLEX_MUL_ASSOC;COMPLEX_MUL_AC]
    THEN REWRITE_TAC[GSYM COMPLEX_POW_MUL;COMPLEX_FIELD `a * --Cx(&1) = --a`]
	THEN ONCE_REWRITE_TAC[COMPLEX_MUL_SYM]
	THEN  ASM_REWRITE_TAC[GSYM complex_div];
    MATCH_MP_TAC(MESON[]`!P Q.P/\Q==> Q`) THEN
	 Pa.EXISTS_TAC " ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n) IN sq_integrable"
	 THEN SUBGOAL_THEN `(((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)  IN sq_integrable /\
 cfun_summable (sq_integrable,r_inprod) (from 0)
 (\n'. Cx (inv (&(FACT n'))) %
       ((ii * Cx pi) % phn sm pow n')
       ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n))) <=> (
	   ((a pow n / Cx (sqrt (&(FACT n)))) % fock sm n)  IN
	   {x|x IN sq_integrable /\ cfun_summable (sq_integrable,r_inprod) (from 0)
	  (\n. Cx (inv(&(FACT n))) % (((ii * Cx pi) % (phn sm)) pow n ) x)})` ASSUME_TAC
	  THENL[REWRITE_TAC[IN_ELIM_THM];ALL_TAC] 
	THEN ASM_REWRITE_TAC[] THEN MATCH_MP_TAC CFUN_SUBSPACE_SMUL
	THEN ASM_SIMP_TAC[IN_ELIM_THM;FOCK_IN_S;cfun_summable;FOCK_PHASE_SHIFT]
	THEN REWRITE_TAC[GSYM cfun_summable] THEN IMP_REWRITE_TAC[ ULINCOP_SUBSPACE];
	RULE_ASSUM_TAC(REWRITE_RULE[IN_ELIM_THM;GSYM CFUN_SUMS_INFSUM])
	  THEN REPEAT (FIRST_X_ASSUM(fun th -> FIRST_ASSUM (fun th1-> ASSUME_TAC ( MATCH_MP th1 th))))
	  THEN ASM_REWRITE_TAC[GSYM CFUN_SUMS_INFSUM]
	  THEN  IMP_REWRITE_TAC[INFSUM_IN_SPC];
	  REWRITE_TAC[CEXP_II_PI;CFUN_SMUL_ASSOC;COMPLEX_POW_MUL;COMPLEX_MUL_RID;COMPLEX_POW_ONE;complex_div;GSYM COMPLEX_MUL_ASSOC;COMPLEX_MUL_AC]
    THEN REWRITE_TAC[GSYM COMPLEX_POW_MUL;COMPLEX_FIELD `a * --Cx(&1) = --a`]
	THEN ONCE_REWRITE_TAC[COMPLEX_MUL_SYM]
	THEN  ASM_REWRITE_TAC[GSYM complex_div]];
	REWRITE_TAC[CEXP_II_PI;CFUN_SMUL_ASSOC;COMPLEX_POW_MUL;COMPLEX_MUL_RID;COMPLEX_POW_ONE;complex_div;GSYM COMPLEX_MUL_ASSOC;COMPLEX_MUL_AC]
    THEN REWRITE_TAC[GSYM COMPLEX_POW_MUL;COMPLEX_FIELD `a * --Cx(&1) = --a`]
	THEN ONCE_REWRITE_TAC[COMPLEX_MUL_SYM]
	THEN  ASM_REWRITE_TAC[GSYM complex_div] THEN
	IMP_REWRITE_TAC[tem INPROD_EQUV_REFL]];
	RULE_ASSUM_TAC(REWRITE_RULE[IN_ELIM_THM;GSYM CFUN_SUMS_INFSUM])
	  THEN REPEAT (FIRST_X_ASSUM(fun th -> FIRST_ASSUM (fun th1-> ASSUME_TAC ( MATCH_MP th1 th))))
	  THEN ASM_REWRITE_TAC[GSYM CFUN_SUMS_INFSUM]
	  THEN  IMP_REWRITE_TAC[INFSUM_IN_SPC]]);;